import java.awt.*;
import java.awt.event.*;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Random;

import javax.swing.*;

/**
 * GameCourt
 * 
 * This class holds the primary game logic for how different objects interact with one another. Take
 * time to understand how the timer interacts with the different methods and how it repaints the GUI
 * on every tick().
 */
@SuppressWarnings("serial")
public class GameCourt extends JPanel {   
    public boolean playing = false; // whether the game is running 
    private JLabel status; // Current status text, i.e. "Running..."
    
    //game components
    private GamePiece[][] board;
    private Food food;
    private ArrayList<Snake> snake;  //collection to create snake
    private Color snakeColor = Color.GREEN;
        
    //stores the scores of players
    public ArrayList<HighScore> highscores = new ArrayList<HighScore>(); 

    // Game constants
    public static final int COURT_WIDTH = 400;
    public static final int COURT_HEIGHT = 400;
    public static final int SNAKE_WIDTH = 10;
    public static final int SNAKE_VELOCITY = 10;

    // Update interval for timer, in milliseconds
    public static final int INTERVAL = 35;
    
    //size of the highscore list
    public static final int NUM_HIGHSCORES = 3;
    
    //score and name of the player
    private int score = 0;
    private String name;
    
    //file i/o stuff
    public FileWriter fw;
    public BufferedWriter bw;
    public FileReader fr;
    public BufferedReader br;

    public GameCourt(JLabel status) {
        // creates border around the court area, JComponent method
        setBorder(BorderFactory.createLineBorder(Color.BLACK));

        // The timer is an object which triggers an action periodically with the given INTERVAL. We
        // register an ActionListener with this timer, whose actionPerformed() method is called each
        // time the timer triggers. We define a helper method called tick() that actually does
        // everything that should be done in a single timestep.
        Timer timer = new Timer(INTERVAL, new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                tick();
            }
        });
        timer.start(); // MAKE SURE TO START THE TIMER!

        // Enable keyboard focus on the court area.
        // When this component has the keyboard focus, key events are handled by its key listener.
        setFocusable(true);

        // This key listener allows the square to move as long as an arrow key is pressed, by
        // changing the square's velocity accordingly. (The tick method below actually moves the
        // square.)
        addKeyListener(new KeyAdapter() {
            public void keyPressed(KeyEvent e) {
                if (e.getKeyCode() == KeyEvent.VK_LEFT) {
                	for (int i = 0; i < snake.size(); i++) {
                		 snake.get(i).setVx(-SNAKE_VELOCITY);
                         snake.get(i).setVy(0);
                	}
                   
                } else if (e.getKeyCode() == KeyEvent.VK_RIGHT) {
                	for (int i = 0; i < snake.size(); i++) {
	               		 snake.get(i).setVx(SNAKE_VELOCITY);
	                     snake.get(i).setVy(0);
	               	}
                } else if (e.getKeyCode() == KeyEvent.VK_DOWN) {
                	for (int i = 0; i < snake.size(); i++) {
               		 	snake.get(i).setVy(SNAKE_VELOCITY);
                        snake.get(i).setVx(0);
                	}
                } else if (e.getKeyCode() == KeyEvent.VK_UP) {
                	for (int i = 0; i < snake.size(); i++) {
               		 	snake.get(i).setVy(-SNAKE_VELOCITY);
               		 	snake.get(i).setVx(0);
                	}
                }
            }
        });

        this.status = status;
        
        this.snake = new ArrayList<Snake>();
        this.board = new GamePiece[COURT_WIDTH / SNAKE_WIDTH][COURT_HEIGHT / SNAKE_WIDTH];
    }

    /**
     * (Re-)set the game to its initial state.
     */
    public void reset() {
    	 for (int i = 0; i< COURT_WIDTH/SNAKE_WIDTH; i++) {
             for(int j = 0; j < COURT_HEIGHT/SNAKE_WIDTH; j++) {
                 board[i][j]  = GamePiece.BOARD;
             }
    	 }
    	 
    	snake = new ArrayList<Snake>();
    	
    	for (int i = 0; i < 3; i++) {
    		snake.add(new Snake(COURT_WIDTH, COURT_HEIGHT, snakeColor, 150 - i*SNAKE_WIDTH, 150));
    	}
    	
    	drawSnake(snake);
        food = new Food(COURT_WIDTH, COURT_HEIGHT, Color.RED);
        score = 0;
        playing = false;
        
        name = JOptionPane.showInputDialog("Enter your name: ");
        
        status.setText("Score: " + score);

        // Make sure that this component has the keyboard focus
        requestFocusInWindow();
    }
    
    
    /**
     * This method is called every time the timer defined in the constructor triggers.
     */
    void tick() {
        if (playing) {    
        	//move and redraw the snake at each timestep
        	for (int i = snake.size() - 1; i > 0; i--) {
        		snake.get(i).setPx(snake.get(i - 1).getPx());
        		snake.get(i).setPy(snake.get(i - 1).getPy());
        	}
        	
        	snake.get(0).move();
        	drawSnake(snake);
        		
            // check for the game end conditions
        	if (snake.get(0).hitWall() != null) {
        		playing = false;
                status.setText("You lose! You ran into the wall!");
                
	            try {
	              fw = new FileWriter("files/highscores.txt", true);
	              bw = new BufferedWriter(fw);
	              bw.write(score + " - " + name);
	              bw.newLine();
	              bw.flush();
	              bw.close();
	            }
	            catch (IOException e) {
	              e.printStackTrace();
	            }    
        	}
        	
        	if (checkSelfIntersect()) {
        		playing = false;
                status.setText("You lose! You ran into yourself!");
                
                try {
                  fw = new FileWriter("files/highscores.txt", true);
                  bw = new BufferedWriter(fw);
                  bw.write(score + " - " + name);
                  bw.newLine();
                  bw.flush();
                  bw.close();
              }
              catch (IOException e) {
                  e.printStackTrace();
              }    
        	}
        	
            if (snake.get(0).intersects(food)) {
            	newFood();
            	score++;
            	status.setText("Score: " + score);
            	snake.add(new Snake(COURT_WIDTH, COURT_HEIGHT, snakeColor, snake.get(snake.size() - 1).getPx(), snake.get(snake.size() - 1).getPy()));
            	drawSnake(snake);
            }

            // update the display
            repaint();
        }
    }
    
    public void newFood() {
    	Random rand = new Random();
    	food.setPx(rand.nextInt(COURT_WIDTH - 2*food.getWidth()) + food.getWidth());
    	food.setPy(rand.nextInt(COURT_HEIGHT - 2*food.getHeight()) + food.getHeight());
    	repaint();
    }

    public void drawSnake(ArrayList<Snake> snake) {
    	for(Snake s : snake) {
    		board[s.getPx()/SNAKE_WIDTH][s.getPy()/SNAKE_WIDTH] = GamePiece.SNAKE;
        }
    }
    
    public boolean checkSelfIntersect() {
    	for (int i = 0; i < snake.size(); i++) {
            for (int j = 0; j < snake.size(); j++) {
                if(snake.get(i).getPx() == snake.get(j).getPx() && snake.get(i).getPy() == snake.get(j).getPy() && i != j) {
                    return true;
                }
            }
        }
    	
    	return false;
    }
    
    @Override
    public void paintComponent(Graphics g) {
        super.paintComponent(g);
        
        food.draw(g);
        
        for (int i = 0; i< COURT_WIDTH/SNAKE_WIDTH; i++) {
        	for(int j = 0; j< COURT_HEIGHT/SNAKE_WIDTH; j++) {
	            if (board[i][j] == GamePiece.SNAKE) {
	            	for (int k = 0; k < snake.size(); k++) {
	            		snake.get(k).draw(g);
	            	}
	            }
        	}
        }
    }
    
    @Override
    public Dimension getPreferredSize() {
        return new Dimension(COURT_WIDTH, COURT_HEIGHT);
    }
    
    public void play() {
    	playing = true;
    	
    	requestFocusInWindow();
    }
    
    public void howto() {
        JFrame jframe = new JFrame("How to Play Snake!");
        jframe.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
        JPanel jpan = new JPanel();
        jpan.setLayout(new BoxLayout(jpan, BoxLayout.Y_AXIS));
        jpan.add(new JLabel("Press the arrow keys to control the snake. \n The goal is to eat the red food, and grow larger. \n If the snake hits the wall or itself, you lose."));
        jframe.setPreferredSize(new Dimension(800, 100));
        jframe.getContentPane().add(jpan, BorderLayout.CENTER);
        jframe.pack();
        jframe.setVisible(true);
        requestFocusInWindow();
    }
    
    public void highscores() {
        highscores = new ArrayList<HighScore>();
        JFrame jframe = new JFrame("Highscores");
        jframe.getContentPane().removeAll();
        jframe.getContentPane().repaint();
        jframe.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
        JPanel jpan = new JPanel();
        jpan.removeAll();
        jpan.repaint();
        jpan.setLayout(new BoxLayout(jpan, BoxLayout.Y_AXIS));
        JLabel hs = new JLabel("High Scores:");
        jframe.setPreferredSize(new Dimension(150, 100));
        jpan.add(hs);
        
        try {
            fr = new FileReader("files/highscores.txt");
            br = new BufferedReader(fr);
            
            String s;
            while((s = br.readLine()) != null) {
                String[] scores = s.split(" - ");
                int score = Integer.parseInt(scores[0]);
                String name = scores[1]; 
                highscores.add(new HighScore(score, name));                           
            } 
            
            br.close();
        }
        catch (IOException e) {
            e.printStackTrace();
        }    
        
        Collections.sort(highscores);
               	
        for (int i = 0; i < highscores.size(); i++) {
        	if (i < NUM_HIGHSCORES) {
        		JLabel s = new JLabel("#" + (i + 1) + " " + highscores.get(i).getHighscore() + " - " + highscores.get(i).getName());
                jpan.add(s);
            }        
        }
            
        jframe.getContentPane().add(jpan, BorderLayout.CENTER);
        jframe.pack();
        jframe.setVisible(true);
        requestFocusInWindow();
    }
}
